Thread (75 messages) 75 messages, 5 authors, 2023-01-24

Re: [PATCH v8 07/10] block: Switch to pinning pages.

From: David Hildenbrand <hidden>
Date: 2023-01-24 14:33:12
Also in: linux-fsdevel, lkml

On 23.01.23 18:30, David Howells wrote:
quoted hunk ↗ jump to hunk
Add BIO_PAGE_PINNED to indicate that the pages in a bio are pinned
(FOLL_PIN) and that the pin will need removing.

Signed-off-by: David Howells <dhowells@redhat.com>
cc: Al Viro <viro@zeniv.linux.org.uk>
cc: Jens Axboe <axboe@kernel.dk>
cc: Jan Kara <jack@suse.cz>
cc: Christoph Hellwig <hch@lst.de>
cc: Matthew Wilcox <willy@infradead.org>
cc: Logan Gunthorpe <logang@deltatee.com>
cc: linux-block@vger.kernel.org
---

Notes:
     ver #8)
      - Move the infrastructure to clean up pinned pages to this patch [hch].
      - Put BIO_PAGE_PINNED before BIO_PAGE_REFFED as the latter should
        probably be removed at some point.  FOLL_PIN can then be renumbered
        first.

  block/bio.c               |  7 ++++---
  block/blk.h               | 28 ++++++++++++++++++++++++++++
  include/linux/bio.h       |  3 ++-
  include/linux/blk_types.h |  1 +
  4 files changed, 35 insertions(+), 4 deletions(-)
diff --git a/block/blk.h b/block/blk.h
index 4c3b3325219a..294044d696e0 100644
--- a/block/blk.h
+++ b/block/blk.h
@@ -425,6 +425,34 @@ int bio_add_hw_page(struct request_queue *q, struct bio *bio,
  		struct page *page, unsigned int len, unsigned int offset,
  		unsigned int max_sectors, bool *same_page);
  
+/*
+ * Set the cleanup mode for a bio from an iterator and the extraction flags.
+ */
+static inline void bio_set_cleanup_mode(struct bio *bio, struct iov_iter *iter)
+{
+	unsigned int cleanup_mode = iov_iter_extract_mode(iter);
+
+	if (cleanup_mode & FOLL_GET)
+		bio_set_flag(bio, BIO_PAGE_REFFED);
+	if (cleanup_mode & FOLL_PIN)
+		bio_set_flag(bio, BIO_PAGE_PINNED);
Can FOLL_GET ever happen?

IOW, can't this even be

if (user_backed_iter(iter))
	bio_set_flag(bio, BIO_PAGE_PINNED);

diff --git a/block/bio.c b/block/bio.c
index 40c2b01906da..6f98bcfc0c92 100644
--- a/block/bio.c
+++ b/block/bio.c
@@ -1170,13 +1170,14 @@ bool bio_add_folio(struct bio *bio, struct folio *folio, size_t len,
  
  void __bio_release_pages(struct bio *bio, bool mark_dirty)
  {
+	unsigned int gup_flags = bio_to_gup_flags(bio);
  	struct bvec_iter_all iter_all;
  	struct bio_vec *bvec;
  
  	bio_for_each_segment_all(bvec, bio, iter_all) {
  		if (mark_dirty && !PageCompound(bvec->bv_page))
  			set_page_dirty_lock(bvec->bv_page);
-		put_page(bvec->bv_page);
+		page_put_unpin(bvec->bv_page, gup_flags);
  	}
  }
  EXPORT_SYMBOL_GPL(__bio_release_pages);
@@ -1496,8 +1497,8 @@ void bio_set_pages_dirty(struct bio *bio)
   * the BIO and re-dirty the pages in process context.
   *
   * It is expected that bio_check_pages_dirty() will wholly own the BIO from
  * here on.  It will run one put_page() against each page and will run one
  * bio_put() against the BIO.
   */
  
  static void bio_dirty_fn(struct work_struct *work);
@@ -1496,8 +1497,8 @@ void bio_set_pages_dirty(struct bio *bio)
   *
   * It is expected that bio_check_pages_dirty() will wholly own the BIO from
- * here on.  It will run one put_page() against each page and will run one
- * bio_put() against the BIO.
+ * here on.  It will run one page_put_unpin() against each page and will run
+ * one bio_put() against the BIO.
   */
  
-- 
Thanks,

David / dhildenb
Keyboard shortcuts
hback out one level
jnext message in thread
kprevious message in thread
ldrill in
Escclose help / fold thread tree
?toggle this help